# -*- coding: utf-8 -*-
# Author: Aethlios <tom.chambaretaud@protonmail.com>

import base64
import re
import concurrent.futures
from tqdm import tqdm

from functools import partial
from typing import Callable, Any, Optional
from decimal import Decimal, InvalidOperation

import shortuuid
import uuid

from resetTolkien.utils import (
    FormatType,
    EncodingType,
    HashingType,
    HashIdentifierType,
    TimestampHashFormat,
    NotAHash,
    GeneratorLen,
    to_microsecond_timestamp,
    from_microsecond_timestamp,
    uniqid,
    deuniqid,
    detect_datetime,
    to_uuidv1,
    from_uuidv1,
    to_mongodb_objectid,
    from_mongodb_objectid,
    urlencode,
    urldecode,
    import_format_with_args,
)
from resetTolkien.constants import (
    DEFAULT_THREAD_NUMBER,
    DATE_SLASH_REGEX,
    DATE_DASH_REGEX,
    TIME_REGEX,
    MONTH_REGEX,
    PARTIAL_FUNC_NAME_REGEX,
    TIMESTAMP_STR_LENGTH,
)
from resetTolkien.hashes import Hashes


class Formatter:
    """Class for encoding, hashing and decrypting a timestamp-based value"""

    ### Utils
    def import_formats(self, formats: list[str]) -> list[FormatType]:
        """Converts a list of strings into a list of corresponding encoding and hash functions."""

        results: list[FormatType] = []
        format_funcs: dict[str, EncodingType | HashingType] = self.allFormat()

        for func_name in formats:
            if (
                "(" in func_name
                and func_name.split("(")[0] in self.allEncodingWithArgs()
            ):
                func, args = import_format_with_args(
                    func_name, self.allEncodingWithArgs()
                )
                format_func = partial(func, **args)
                results.append(format_func)
            else:
                if not func_name in format_funcs:
                    raise ValueError(f"{func_name} is not in available formats")
                results.append(format_funcs[func_name])
        return results

    def export_formats(self, formats: list[FormatType]) -> str:
        """Converts a list of encoding and hash functions into a list of strings."""

        formats_name: list[str] = []
        for format in formats:
            if isinstance(format, partial):
                elems = re.search(PARTIAL_FUNC_NAME_REGEX, str(format))
                if not elems:
                    raise ValueError(f"No partial name found : {format}")
                args = [
                    elem
                    for elem in elems.groups()[1].split(",")
                    if elem.split("=")[1] != "None"
                ]
                for arg in args:
                    name = f"{elems.groups()[0]}({arg})"
                    formats_name.append(name)
            else:
                formats_name.append(str(format.__name__))
        return ",".join(formats_name)

    def isLiteralIntegerOrFloat(self, token: str) -> bool:
        """Confirms that the value is numeric."""

        try:
            Decimal(token)
            return True
        except InvalidOperation:
            return False

    def getNumbers(self, token: str) -> list[str]:
        """Extracts numeric values from the string value."""

        matches = re.findall(r"([0-9]+\.?[0-9]*)", str(token))
        return [match for match in matches if self.isLiteralIntegerOrFloat(match)]

    def searchTimestamps(self, numbers: list[str]) -> list[Decimal | int]:
        """Extracts nearby timestamp values from numerical values."""

        matches: list[Decimal | int] = []
        for number in numbers:
            if self.isLiteralIntegerOrFloat(number):
                if "." in number:
                    n = Decimal(number)
                elif len(number) > TIMESTAMP_STR_LENGTH:
                    n = Decimal(
                        number[:TIMESTAMP_STR_LENGTH]
                        + "."
                        + number[TIMESTAMP_STR_LENGTH:]
                    )
                else:
                    n = int(number)
                try:
                    from_microsecond_timestamp(n, verify=True)
                except ValueError:
                    continue
                matches.append(n)
        return matches

    def isDatetime(self, token: str) -> bool:
        """Confirms that the value is datetime format."""

        return (
            re.search(DATE_SLASH_REGEX, token) != None
            or re.search(DATE_DASH_REGEX, token) != None
            or re.search(TIME_REGEX, token) != None
            or re.search(MONTH_REGEX, token) != None
        )

    ### Format

    def allFormat(self) -> dict[str, EncodingType | HashingType]:
        """Returns all functions."""

        formats: dict[str, EncodingType | HashingType] = {}
        formats.update(self.allEncoding())
        formats.update(self.allHashing())
        return formats

    ### Encoding

    def allEncoding(self) -> dict[str, EncodingType]:
        """Returns all encoding functions."""

        return {
            "prefix_suffix": self.prefix_suffix,
            "base32": self.base32,
            "base64": self.base64,
            "urlencode": self.urlencode,
            "hexint": self.hexint,
            "hexstr": self.hexstr,
            "uniqid": self.uniqid,
            "shortuuid": self.shortuuid,
            "uuidv1": self.uuidv1,
            "mongodb_objectid": self.mongodb_objectid,
            "datetime": self.datetime,
            "datetimeRFC2822": self.datetimeRFC2822,
        }

    def allEncodingWithArgs(self) -> dict[str, EncodingType]:
        """Returns all encoding functions with required arguments."""

        return {"prefix_suffix": self.prefix_suffix}

    def availableEncoding(self) -> dict[str, EncodingType]:
        """Returns the encoding functions available to try out."""

        return {
            "base32": self.base32,
            "base64": self.base64,
            "urlencode": self.urlencode,
            "hexint": self.hexint,
            "hexstr": self.hexstr,
            "uniqid": self.uniqid,
            "shortuuid": self.shortuuid,
            "mongodb_objectid": self.mongodb_objectid,
            "uuidv1": self.uuidv1,
        }

    ### Encoding function list

    def prefix_suffix(
        self, token: str, encode: Optional[bool] = True, **kwargs: Any
    ) -> str:
        """Adds prefix and suffix if provided to a string value."""

        if encode:
            prefix = ""
            if "prefix" in kwargs and kwargs["prefix"]:
                prefix = kwargs["prefix"]
            suffix = ""
            if "suffix" in kwargs and kwargs["suffix"]:
                suffix = kwargs["suffix"]
            return f"{prefix}{token}{suffix}"
        raise ValueError("No possible to decode with prefix_suffix function.")

    def datetime(self, token: str, encode: Optional[bool] = True, **kwargs: Any) -> str:
        """Converts from a timestamp to a date or vice versa"""

        if encode:
            timezone = kwargs["timezone"]
            date_format_of_token = kwargs["date_format_of_token"]
            return from_microsecond_timestamp(
                Decimal(token),
                timezone=timezone,
                date_format_of_token=date_format_of_token,
            )
        return str(detect_datetime(token))

    def datetimeRFC2822(
        self, token: str, encode: Optional[bool] = True, **kwargs: Any
    ) -> str:
        """Converts from a string RFC2822 datetime to a date or vice versa"""

        if encode:
            timezone = kwargs["timezone"]
            date = from_microsecond_timestamp(
                Decimal(token),
                timezone=timezone,
                date_format_of_token="%a, %d %b %Y %H:%M:%S %Z",
            )
            # No existing method with datetime to get the RFC2822, so we go with regex :
            # "UTC(+|-)07:00" -> "(+|-)0700"
            date = re.sub(r"UTC(.)(\d{2}):(\d{2})", r"\1\2\3", date)
            # "UTC" -> "+0000"
            date = re.sub(r"UTC$", r"+0000", date)
            return date
        return str(detect_datetime(token))

    def timestamp(
        self, token: str, encode: Optional[bool] = True, **kwargs: Any
    ) -> str:
        """Check if it's a timestamp value"""

        if encode:
            return to_microsecond_timestamp(token)
        return from_microsecond_timestamp(Decimal(token))

    def mongodb_objectid(
        self, token: str, encode: Optional[bool] = True, **kwargs: Any
    ) -> str:
        """Converts from an PostgreSQL to a timestamp or vice versa"""

        if encode:
            init_token = kwargs["init_token"]
            return to_mongodb_objectid(token, init_token)
        return from_mongodb_objectid(token)

    def uuidv1(self, token: str, encode: Optional[bool] = True, **kwargs: Any) -> str:
        """Converts from an UUID to a timestamp or vice versa"""

        if encode:
            init_token = kwargs["init_token"]
            return to_uuidv1(token, init_token)
        return from_uuidv1(token)

    def base32(self, token: str, encode: Optional[bool] = True, **kwargs: Any) -> str:
        """Converts from a base32-encoded value to a string or vice versa"""

        if encode:
            return base64.b32encode(str(token).encode()).decode()
        return base64.b32decode(str(token).encode()).decode()

    def base64(self, token: str, encode: Optional[bool] = True, **kwargs: Any) -> str:
        """Converts from a base64-encoded value to a string or vice versa"""

        if encode:
            return base64.b64encode(str(token).encode()).decode()
        return base64.b64decode(str(token).encode()).decode()

    def urlencode(
        self, token: str, encode: Optional[bool] = True, **kwargs: Any
    ) -> str:
        """Converts from a URL-encoded value to a string or vice versa"""

        if encode:
            return urlencode(token)
        return urldecode(token)

    def shortuuid(
        self, token: str, encode: Optional[bool] = True, **kwargs: Any
    ) -> str:
        """Converts from a shortuuid value to a UUID or vice versa"""

        if encode:
            return shortuuid.encode(uuid.UUID(str(token)))
        return str(shortuuid.decode(token))

    def uniqid(self, token: str, encode: Optional[bool] = True, **kwargs: Any) -> str:
        """Converts from a uniqid value to a timestamp or vice versa"""

        if encode:
            return uniqid(Decimal(token))
        return deuniqid(token)

    def hexint(self, token: str, encode: Optional[bool] = True, **kwargs: Any) -> str:
        """Converts from a int-hex value to a timestamp or vice versa"""

        if encode:
            return hex(int(Decimal(token))).replace("0x", "")
        return str(int(token, 16))

    def hexstr(self, token: str, encode: Optional[bool] = True, **kwargs: Any) -> str:
        """Converts from a str-hex value to a timestamp or vice versa"""

        if encode:
            return str(token).encode().hex()
        return bytes.fromhex(str(token)).decode()

    # Hashing

    def allHashIdentifiers(self) -> dict[str, HashIdentifierType]:
        """Returns all hash identifier functions."""

        return {
            "md5": self.is_md5,
            "sha1": self.is_sha1,
            "sha224": self.is_sha224,
            "sha256": self.is_sha256,
            "sha384": self.is_sha384,
            "sha512": self.is_sha512,
            "sha3_224": self.is_sha3_224,
            "sha3_256": self.is_sha3_256,
            "sha3_384": self.is_sha3_384,
            "sha3_512": self.is_sha3_512,
            "blake_256": self.is_blake_256,
            "blake_512": self.is_blake_512,
        }

    def allHashing(self) -> dict[str, HashingType]:
        """Returns all hash functions in dict with name as key."""

        return {
            "md5": self.md5,
            "sha1": self.sha1,
            "sha224": self.sha224,
            "sha256": self.sha256,
            "sha384": self.sha384,
            "sha512": self.sha512,
            "sha3_224": self.sha3_224,
            "sha3_256": self.sha3_256,
            "sha3_384": self.sha3_384,
            "sha3_512": self.sha3_512,
            "blake_256": self.blake_256,
            "blake_512": self.blake_512,
        }

    def availableHashingDict(
        self, token: Optional[str] = None
    ) -> dict[str, HashingType]:
        """Returns the hash functions available by identifying the input hash."""

        if not token:
            return self.allHashing()
        hashes: dict[str, HashingType] = {}
        identifiers = self.allHashIdentifiers()
        for h in identifiers:
            is_hash = identifiers[h]
            hash = is_hash(str(token))
            if hash != None:
                hashes[h] = hash
        return hashes

    def encode(
        self,
        value: str,
        token: Optional[str],
        formats: list[FormatType],
        timezone: int = 0,
        date_format_of_token: Optional[str] = None,
    ) -> str:
        for format in formats[::-1]:
            token = format(
                value,
                encode=True,
                timezone=timezone,
                date_format_of_token=date_format_of_token,
                init_token=token,
            )
            if isinstance(token, str):
                value = token
            else:
                raise ValueError("The token has not been encoded but decoded")
        return value

    def hashing_with_prefix(
        self,
        hashes: list[HashingType],
        timestamp_hash_formats: list[TimestampHashFormat],
        prefixes: list[str],
        suffixes: list[str],
        token: str,
        timezone: int,
        date_format_of_token: Optional[str],
        alternative_tokens: list[str],
        values: tuple[str, str],
    ) -> tuple[
        Optional[str],
        Optional[str],
        Optional[str],
        Optional[HashingType],
        Optional[TimestampHashFormat],
    ]:
        """Returns timestamp and encoded value if hashed value from various provided prefixes and suffixes"""

        timestamp, _ = values
        if token not in alternative_tokens:
            alternative_tokens.append(token)
        for timestamp_hash_format in timestamp_hash_formats:
            for alternative_token in alternative_tokens:
                encoded_timestamp = self.encode(
                    timestamp,
                    alternative_token,
                    timestamp_hash_format.formats_output,
                    timezone=timezone,
                    date_format_of_token=date_format_of_token,
                )
                for hash_func in hashes:
                    if token == hash_func(encoded_timestamp):
                        return timestamp, None, None, hash_func, timestamp_hash_format
                    for prefix in prefixes:
                        value = "%s%s" % (prefix, encoded_timestamp)
                        if token == hash_func(value):
                            return timestamp, prefix, None, hash_func, timestamp_hash_format
                    for suffix in suffixes:
                        value = "%s%s" % (encoded_timestamp, suffix)
                        if token == hash_func(value):
                            return timestamp, None, suffix, hash_func, timestamp_hash_format
                    for prefix in prefixes:
                        for suffix in suffixes:
                            value = "%s%s%s" % (prefix, encoded_timestamp, suffix)
                            if token == hash_func(value):
                                return (
                                    timestamp,
                                    prefix,
                                    suffix,
                                    hash_func,
                                    timestamp_hash_format,
                                )
        return None, None, None, None, None

    def multithread_decrypt(
        self,
        token: str,
        possibleTokens: GeneratorLen,
        timestamp_hash_formats: list[TimestampHashFormat],
        hashes: list[HashingType],
        prefixes: list[str],
        suffixes: list[str],
        timezone: int,
        date_format_of_token: Optional[str],
        alternative_tokens: list[str],
        nb_threads: int = DEFAULT_THREAD_NUMBER,
        progress_active: bool = False,
    ) -> tuple[
        Optional[str],
        Optional[str],
        Optional[str],
        Optional[HashingType],
        Optional[TimestampHashFormat],
    ]:
        """Decrypts a timestamp-based value by using multithreading"""

        data = None, None, None, None, None
        chunksize = round(pow(len(possibleTokens) / nb_threads, 0.5))

        with tqdm(total=len(possibleTokens), disable=(not progress_active)) as progress:
            with concurrent.futures.ProcessPoolExecutor(
                max_workers=nb_threads
            ) as executor:
                for (
                    timestamp,
                    prefix,
                    suffix,
                    hash,
                    timestamp_hash_format,
                ) in executor.map(
                    partial(
                        self.hashing_with_prefix,
                        hashes,
                        timestamp_hash_formats,
                        prefixes,
                        suffixes,
                        token,
                        timezone,
                        date_format_of_token,
                        alternative_tokens,
                    ),
                    possibleTokens,
                    chunksize=chunksize,
                ):
                    if timestamp:
                        data = timestamp, prefix, suffix, hash, timestamp_hash_format
                        break
                    progress.update(1)
        return data

    def native_decrypt(
        self,
        token: str,
        possibleTokens: GeneratorLen,
        timestamp_hash_formats: list[TimestampHashFormat],
        hashes: list[HashingType],
        prefixes: list[str],
        suffixes: list[str],
        timezone: int,
        date_format_of_token: Optional[str],
        alternative_tokens: list[str],
        progress_active: bool = False,
    ) -> tuple[
        Optional[str],
        Optional[str],
        Optional[str],
        Optional[HashingType],
        Optional[TimestampHashFormat],
    ]:
        """Decrypts a timestamp-based value by using naive method"""

        with tqdm(total=len(possibleTokens), disable=(not progress_active)) as progress:
            for values in possibleTokens:
                timestamp, prefix, suffix, hash, timestamp_hash_format = (
                    self.hashing_with_prefix(
                        hashes,
                        timestamp_hash_formats,
                        prefixes,
                        suffixes,
                        token,
                        timezone,
                        date_format_of_token,
                        alternative_tokens,
                        values,
                    )
                )
                if timestamp:
                    return timestamp, prefix, suffix, hash, timestamp_hash_format
                progress.update(1)
        raise NotAHash(f"It is not a hash")

    def is_hash(self, hash: str, hash_func: HashingType) -> Optional[HashingType]:
        """Generic function to detect a hash value"""

        if (
            hash.isdigit() == False
            and hash.isalpha() == False
            and hash.isalnum() == True
            and len(hash) == len(hash_func("FIXED_VALUE", encode=True))
        ):
            return hash_func
        return None

    def hash(
        self,
        hash_func: Callable[[str], str],
        token: str,
    ) -> str:
        """Generic function for hashing or decrypting a value based on a timestamp."""
        return hash_func(token)

    ### Hash function list

    def is_md5(self, hash: str) -> Optional[HashingType]:
        return self.is_hash(hash, self.md5)

    def md5(self, token: str, **kwargs: Any) -> str:
        return self.hash(Hashes.md5, token)

    def is_sha1(self, hash: str) -> Optional[HashingType]:
        return self.is_hash(hash, self.sha1)

    def sha1(self, token: str, **kwargs: Any) -> str:
        return self.hash(Hashes.sha1, token)

    def is_sha224(self, hash: str) -> Optional[HashingType]:
        return self.is_hash(hash, self.sha224)

    def sha224(self, token: str, **kwargs: Any) -> str:
        return self.hash(Hashes.sha224, token)

    def is_sha256(self, hash: str) -> Optional[HashingType]:
        return self.is_hash(hash, self.sha256)

    def sha256(self, token: str, **kwargs: Any) -> str:
        return self.hash(Hashes.sha256, token)

    def is_sha384(self, hash: str) -> Optional[HashingType]:
        return self.is_hash(hash, self.sha384)

    def sha384(self, token: str, **kwargs: Any) -> str:
        return self.hash(Hashes.sha384, token)

    def is_sha512(self, hash: str) -> Optional[HashingType]:
        return self.is_hash(hash, self.sha512)

    def sha512(self, token: str, **kwargs: Any) -> str:
        return self.hash(Hashes.sha512, token)

    def is_sha3_224(self, hash: str) -> Optional[HashingType]:
        return self.is_hash(hash, self.sha3_224)

    def sha3_224(self, token: str, **kwargs: Any) -> str:
        return self.hash(Hashes.sha3_224, token)

    def is_sha3_256(self, hash: str) -> Optional[HashingType]:
        return self.is_hash(hash, self.sha3_256)

    def sha3_256(self, token: str, **kwargs: Any) -> str:
        return self.hash(Hashes.sha3_256, token)

    def is_sha3_384(self, hash: str) -> Optional[HashingType]:
        return self.is_hash(hash, self.sha3_384)

    def sha3_384(self, token: str, **kwargs: Any) -> str:
        return self.hash(Hashes.sha3_384, token)

    def is_sha3_512(self, hash: str) -> Optional[HashingType]:
        return self.is_hash(hash, self.sha3_512)

    def sha3_512(self, token: str, **kwargs: Any) -> str:
        return self.hash(Hashes.sha3_512, token)

    def is_blake_256(self, hash: str) -> Optional[HashingType]:
        return self.is_hash(hash, self.blake_256)

    def blake_256(self, token: str, **kwargs: Any) -> str:
        return self.hash(Hashes.blake_256, token)

    def is_blake_512(self, hash: str) -> Optional[HashingType]:
        return self.is_hash(hash, self.blake_512)

    def blake_512(self, token: str, **kwargs: Any) -> str:
        return self.hash(Hashes.blake_512, token)



# -*- coding: utf-8 -*-
# Author: Aethlios <tom.chambaretaud@protonmail.com>

import argparse
import datetime
from decimal import Decimal

from resetTolkien.resetTolkien import ResetTolkien
from resetTolkien import version
from resetTolkien.format import Formatter
from resetTolkien.utils import SplitArgs, SERVER_DATE_FORMAT, server_date_example
from resetTolkien.constants import (
    OUTPUT_STRINGS,
    PROG_DESCRIPTION,
    DEFAULT_THREAD_NUMBER,
    MIN_DEPTH_LEVEL,
    MAX_DEPTH_LEVEL,
    DEFAULT_VERBOSITY_LEVEL,
    MAX_VERBOSITY_LEVEL,
    DEFAULT_DEPTH_LEVEL,
    DEFAULT_DECIMAL_LENGTH,
    DEFAULT_TIMERANGE_FOR_INT_TIMESTAMP,
    DEFAULT_TIMERANGE_FOR_FLOAT_TIMESTAMP,
    DEFAULT_CONFIG_FILE,
)

# PARENT

parent_parser = argparse.ArgumentParser(add_help=False)
parent_parser.add_argument(
    "-r",
    "--roleplay",
    help="Not recommended if you don't have anything else to do",
    action="store_true",
)
parent_parser.add_argument(
    "-v",
    "--verbosity",
    help=f"Verbosity level (default: {DEFAULT_VERBOSITY_LEVEL})",
    type=int,
    choices=range(DEFAULT_VERBOSITY_LEVEL, MAX_VERBOSITY_LEVEL + 1),
    default=DEFAULT_VERBOSITY_LEVEL,
)
parent_parser.add_argument(
    "-c",
    "--config",
    help=f"Config file to set TimestampHashFormat (default: {DEFAULT_CONFIG_FILE})",
    type=str,
    default=DEFAULT_CONFIG_FILE,
)
parent_parser.add_argument(
    "--threads",
    help=f"Define the number of parallelized tasks for the decryption attack on the hash. (default: {DEFAULT_THREAD_NUMBER})",
    type=int,
    default=DEFAULT_THREAD_NUMBER,
)
parent_parser.add_argument(
    "--date-format-of-token",
    help="Date format for the token - please set it if you have found a date as input.",
)
parent_parser.add_argument(
    "--only-int-timestamp",
    help="Only use integer timestamp. (default: False)",
    action="store_true",
)
parent_parser.add_argument(
    "--decimal-length",
    help=f"Length of the float timestamp (default: {DEFAULT_DECIMAL_LENGTH})",
    type=int,
)
parent_parser.add_argument(
    "--int-timestamp-range",
    help=f"Time range over which the int timestamp will be tested before and after the input value (default: {DEFAULT_TIMERANGE_FOR_INT_TIMESTAMP}s)",
    type=int,
)
parent_parser.add_argument(
    "--float-timestamp-range",
    help=f"Time range over which the float timestamp will be tested before and after the input value (default: {DEFAULT_TIMERANGE_FOR_FLOAT_TIMESTAMP}s)",
    type=int,
)
parent_parser.add_argument(
    "--timezone",
    help=f"Timezone of the application for datetime value (default: 0)",
    type=int,
    default=0,
)
parent_parser.add_argument(
    "--progress",
    help="Show a progress bar. (default: True)",
    action="store_true",
    default=True,
)

# MAIN

main_parser = argparse.ArgumentParser(description=PROG_DESCRIPTION)
action_subparser = main_parser.add_subparsers(title="action", dest="action")

main_parser.add_argument(
    "-v", "--version", help="Print tool version", action="store_true"
)

# DETECT

detect_parser = action_subparser.add_parser(
    "detect", help="Detect the format of reset token", parents=[parent_parser]
)
detect_parser.add_argument("token", help="The token given as input.")
detect_parser.add_argument(
    "-l",
    "--level",
    help=f"Level of search depth (default: {DEFAULT_DEPTH_LEVEL})",
    type=int,
    choices=range(MIN_DEPTH_LEVEL, MAX_DEPTH_LEVEL + 1),
    default=DEFAULT_DEPTH_LEVEL,
)
detect_parser.add_argument(
    "-t", "--timestamp", help="The timestamp of the reset request", type=Decimal
)
detect_parser.add_argument(
    "-d",
    "--datetime",
    help=f"The datetime of the reset request",
    type=str,
)
detect_parser.add_argument(
    "--datetime-format",
    help=f'The input datetime format (default: server date format like "{server_date_example}")',
    default=SERVER_DATE_FORMAT,
)
detect_parser.add_argument(
    "--prefixes",
    action=SplitArgs,
    help="List of possible values for the prefix concatenated with the timestamp. Format: prefix1,prefix2",
    default=[],
)
detect_parser.add_argument(
    "--suffixes",
    action=SplitArgs,
    help="List of possible values for the suffix concatenated with the timestamp. Format: suffix1,suffix2",
    default=[],
)
detect_parser.add_argument(
    "--hashes",
    action=SplitArgs,
    help="List of possible hashes to try to detect the format. Format: hash1,hash2 (default: all identified hash)",
    default=[],
)
detect_parser.add_argument(
    "--alternative-tokens",
    action=SplitArgs,
    help="List of possible tokens to try to detect the format with different static data. Format: token1,token2",
    default=[],
)


# BRUTEFORCE

bruteforce_parser = action_subparser.add_parser(
    "bruteforce", help="Attack the reset token", parents=[parent_parser]
)
bruteforce_parser.add_argument("token", help="The token given as input.")
bruteforce_parser.add_argument(
    "-t",
    "--timestamp",
    help="The timestamp of the reset request with victim email",
    type=Decimal,
)
bruteforce_parser.add_argument(
    "-d",
    "--datetime",
    help=f"The datetime of the reset request with victim email",
    type=str,
)
bruteforce_parser.add_argument(
    "--datetime-format",
    help=f'The input datetime format (default: server date format like "{server_date_example}")',
    default=SERVER_DATE_FORMAT,
)
bruteforce_parser.add_argument(
    "--token-format",
    action=SplitArgs,
    help="The token encoding/hashing format - Format: encoding1,encoding2",
)
bruteforce_parser.add_argument(
    "--prefix",
    help="The prefix value concatenated with the timestamp.",
    type=str,
)
bruteforce_parser.add_argument(
    "--suffix",
    help="The suffix value concatenated with the timestamp.",
    type=str,
)
bruteforce_parser.add_argument(
    "-o",
    "--output",
    help="The filename of the output",
    type=argparse.FileType("w", encoding="utf-8"),
)
bruteforce_parser.add_argument(
    "--with-timestamp",
    help="Write the output with timestamp",
    action="store_true",
)
bruteforce_parser.add_argument(
    "--alternative-tokens",
    action=SplitArgs,
    help="List of possible tokens to try to detect the format with different static data. Format: token1,token2",
    default=[],
)

# SANDWICH

sandwich_parser = action_subparser.add_parser(
    "sandwich",
    help="Attack the reset token with sandwich method",
    parents=[parent_parser],
)
sandwich_parser.add_argument("token", help="The token given as input.")
sandwich_parser.add_argument(
    "-bt",
    "--begin-timestamp",
    help="The begin timestamp of the reset request with victim email",
    type=Decimal,
)
sandwich_parser.add_argument(
    "-et",
    "--end-timestamp",
    help="The end timestamp of the reset request with victim email",
    type=Decimal,
)
sandwich_parser.add_argument(
    "-bd",
    "--begin-datetime",
    help="The begin datetime of the reset request with victim email",
    type=str,
)
sandwich_parser.add_argument(
    "-ed",
    "--end-datetime",
    help="The end datetime of the reset request with victim email",
    type=str,
)
sandwich_parser.add_argument(
    "--datetime-format",
    help=f'The input datetime format (default: server date format like "{server_date_example}")',
    default=SERVER_DATE_FORMAT,
)
sandwich_parser.add_argument(
    "--token-format",
    action=SplitArgs,
    help="The token encoding/hashing format - Format: encoding1,encoding2",
)
sandwich_parser.add_argument(
    "--prefix",
    help="The prefix value concatenated with the timestamp.",
    type=str,
)
sandwich_parser.add_argument(
    "--suffix",
    help="The suffix value concatenated with the timestamp.",
    type=str,
)
sandwich_parser.add_argument(
    "-o",
    "--output",
    help="The filename of the output",
    type=argparse.FileType("w", encoding="utf-8"),
)
sandwich_parser.add_argument(
    "--with-timestamp",
    help="Write the output with timestamp",
    action="store_true",
)
sandwich_parser.add_argument(
    "--alternative-tokens",
    action=SplitArgs,
    help="List of possible tokens to try to detect the format with different static data. Format: token1,token2",
    default=[],
)


def main():
    args = main_parser.parse_args()

    if args.version:
        print(version)
        exit()

    if not args.action:
        main_parser.print_help()
        exit()

    output_version = "NOJOKE"

    if args.roleplay:
        output_version = "ROLEPLAY"

    if args.action == "detect":

        if args.datetime:
            args.timestamp = Decimal.from_float(
                datetime.datetime.strptime(args.datetime, args.datetime_format)
                .replace(tzinfo=datetime.timezone.utc)
                .timestamp()
            )

        if args.only_int_timestamp:
            args.timestamp = int(args.timestamp)

        if args.date_format_of_token and (
            len(args.prefixes) != 0 or len(args.suffixes) != 0
        ):
            print(
                "Unsupported usage of prefix/suffix with date format -> Please define this prefix/suffix in the date format."
            )
            exit()

        tolkien = ResetTolkien(
            token=args.token,
            level=args.level,
            timezone=args.timezone,
            timestamp_hash_formats_config_file=args.config,
            decimal_length=args.decimal_length,
            int_range_limit=args.int_timestamp_range,
            float_range_limit=args.float_timestamp_range,
            prefixes=args.prefixes,
            suffixes=args.suffixes,
            hashes=args.hashes,
            date_format_of_token=args.date_format_of_token,
            verbosity=args.verbosity,
            progress_active=args.progress,
            alternative_tokens=args.alternative_tokens,
        )

        results = tolkien.detectFormat(
            timestamp=args.timestamp, nb_threads=args.threads
        )
        success = False

        if results:
            for values, formats, isBasedOnTimestamp in results:
                if isBasedOnTimestamp:
                    success = True
                    print(
                        OUTPUT_STRINGS[output_version]["TIMESTAMP_FOUND"].format(
                            timestamp=values[0], prefix=values[1], suffix=values[2]
                        )
                    )
                    print(
                        OUTPUT_STRINGS[output_version]["TOKEN_FORMAT"].format(
                            formats=Formatter().export_formats(formats)
                        )
                    )
                    if int(Decimal(values[0])) == Decimal(values[0]):
                        print(OUTPUT_STRINGS[output_version]["INT_TIMESTAMP"])

        if not success:
            print(OUTPUT_STRINGS[output_version]["FAIL"])
            exit()

    if args.action == "bruteforce":

        if args.datetime:
            args.timestamp = Decimal.from_float(
                datetime.datetime.strptime(args.datetime, args.datetime_format)
                .replace(tzinfo=datetime.timezone.utc)
                .timestamp()
            )

        if not args.timestamp:
            print(f"Please provide a timestamp via -t or -d arguments.")
            exit()

        if args.only_int_timestamp:
            args.timestamp = int(args.timestamp)

        tolkien = ResetTolkien(
            token=args.token,
            timezone=args.timezone,
            timestamp_hash_formats_config_file=args.config,
            decimal_length=args.decimal_length,
            int_range_limit=args.int_timestamp_range,
            float_range_limit=args.float_timestamp_range,
            date_format_of_token=args.date_format_of_token,
            verbosity=args.verbosity,
            formats=args.token_format,
            progress_active=args.progress,
            alternative_tokens=args.alternative_tokens,
        )

        for token, value in tolkien.generate_possible_token(
            args.timestamp,
            prefix=args.prefix,
            suffix=args.suffix,
            formats=tolkien.formats,
        ):
            output = f"{token}"
            if args.with_timestamp:
                output = f"{value}:{token}"
            if args.output:
                args.output.write(f"{output}\n")
            else:
                print(f"{output}")

        if args.output:
            print(
                OUTPUT_STRINGS[output_version]["OUTPUT_IN_FILE"].format(
                    output=args.output.name
                )
            )

    if args.action == "sandwich":

        if args.begin_datetime:
            args.begin_timestamp = Decimal.from_float(
                datetime.datetime.strptime(args.begin_datetime, args.datetime_format)
                .replace(tzinfo=datetime.timezone.utc)
                .timestamp()
            )

        if args.end_datetime:
            args.end_timestamp = Decimal.from_float(
                datetime.datetime.strptime(args.end_datetime, args.datetime_format)
                .replace(tzinfo=datetime.timezone.utc)
                .timestamp()
            )

        if not args.begin_timestamp:
            print(
                "Please provide a begin timestamp via --begin-timestamp or --begin-datetime arguments."
            )
            exit()

        if not args.end_timestamp:
            print(
                "Please provide a end timestamp via --end-timestamp or --end-datetime arguments."
            )
            exit()

        if args.only_int_timestamp:
            args.begin_timestamp = int(args.begin_timestamp)
            args.end_timestamp = int(args.end_timestamp)

        if args.end_timestamp < args.begin_timestamp:
            print("Please define a beginning of timestamp before the end of timestamp.")
            exit()

        tolkien = ResetTolkien(
            token=args.token,
            timezone=args.timezone,
            timestamp_hash_formats_config_file=args.config,
            decimal_length=args.decimal_length,
            int_range_limit=args.int_timestamp_range,
            float_range_limit=args.float_timestamp_range,
            date_format_of_token=args.date_format_of_token,
            verbosity=args.verbosity,
            formats=args.token_format,
            progress_active=args.progress,
            alternative_tokens=args.alternative_tokens,
        )

        for token, value in tolkien.generate_bounded_possible_token(
            args.begin_timestamp,
            args.end_timestamp,
            prefix=args.prefix,
            suffix=args.suffix,
            formats=tolkien.formats,
        ):
            output = f"{token}"
            if args.with_timestamp:
                output = f"{value}:{token}"
            if args.output:
                args.output.write(f"{output}\n")
            else:
                print(f"{output}")

        if args.output:
            print(
                OUTPUT_STRINGS[output_version]["OUTPUT_IN_FILE"].format(
                    output=args.output.name
                )
            )


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Author: Aethlios <tom.chambaretaud@protonmail.com>

import datetime
import base64
from binascii import hexlify
import hashlib
import uuid
import time
from decimal import Decimal, getcontext

from resetTolkien.resetTolkien import ResetTolkien, FormatType
from resetTolkien.utils import uuid1, uniqid, urlencode, MongoDBObjectID
from resetTolkien.constants import TIMESTAMP_STR_LENGTH, UUID_DECIMAL_LENGTH

import sys

verbosity = 0
if len(sys.argv) == 3 and sys.argv[1] == "-v":
    verbosity = int(sys.argv[2])

threads = 8
DECIMAL_LENGTH = UUID_DECIMAL_LENGTH

getcontext().prec = TIMESTAMP_STR_LENGTH + DECIMAL_LENGTH

TIMEDELTA_WITH_FLOAT_VALUE = Decimal(0.1)
TIMEDELTA_WITH_INT_VALUE = 30

OK = "\033[92mOK\033[0m"
NOK = "\033[91mNOK\033[0m"
PROGRESS_ACTIVE = True


def benchmark_multithread() -> None:
    timestamp_input = getFloatTimestamp()
    token = hashlib.md5("value".encode()).hexdigest()
    print("native")
    tolkien = ResetTolkien(
        token=token,
        verbosity=2,
    )
    t = time.time()
    result = tolkien.detectFormat(timestamp=timestamp_input, nb_threads=1)
    t2 = time.time()
    print(result)
    print(f"Time : {t2-t}")
    print("multithread")
    tolkien = ResetTolkien(
        token=token,
        verbosity=2,
        prefixes=["a", "c"],
        suffixes=["b", "d"],
    )
    t = time.time()
    result = tolkien.detectFormat(timestamp=timestamp_input, nb_threads=8)
    t2 = time.time()
    print(result)
    print(f"Time : {t2-t}")


def _check(
    tolkien: ResetTolkien,
    value: str,
    init_token: str,
    results: list[tuple[tuple[str, str | None, str | None], list[FormatType], bool]],
) -> tuple[bool, list[list[FormatType]]]:
    for _, formats, isBasedOnTimestamp in results:
        if isBasedOnTimestamp:
            try:
                result = tolkien.encode(value, formats=formats)
                if result != init_token and len(results) > 0:
                    print(formats)
                    print(
                        f"result : {result} - expected result : {init_token} - value : {value}"
                    )
                return (
                    result == init_token,
                    [formats for _, formats, _ in results if isBasedOnTimestamp],
                )
            except:
                pass
    return (False, [])


def check(
    value: str | Decimal | int,
    init_token: str,
    description: str = "Generic test",
    timestamp_input: Decimal | int | None = None,
    prefixes: list[str] | None = None,
    suffixes: list[str] | None = None,
    date_format_of_token: str | None = None,
    timezone: int = 0,
) -> None:
    prefixes = prefixes if prefixes else []
    suffixes = suffixes if suffixes else []

    if verbosity >= 1:
        print(f"Value : {value}")
        print(f"Token : {init_token}")

    if isinstance(value, int) or isinstance(value, Decimal):
        value = str(value)

    tolkien = ResetTolkien(
        token=init_token,
        timezone=timezone,
        verbosity=verbosity,
        prefixes=prefixes,
        suffixes=suffixes,
        date_format_of_token=date_format_of_token,
        decimal_length=DECIMAL_LENGTH,
        progress_active=PROGRESS_ACTIVE,
    )
    start = time.time()
    results = tolkien.detectFormat(timestamp=timestamp_input, nb_threads=threads)
    end = time.time()
    if not results:
        print(f"[{round(end - start, 3)}s] {description} : {NOK}")
        raise Exception("No defined token")
    if verbosity >= 1:
        print(f"Partial results : {results}")
    success, possible_formats = _check(tolkien, value, init_token, results)
    print(
        f"[{round(end - start, 3)}s] {description} : {(OK if success else NOK)} -> {results[0][0][0]} (possibles formats : {len(possible_formats)})"
    )


def getIntTimestamp() -> int:
    return int(datetime.datetime.now().timestamp())


def getFloatTimestamp() -> Decimal:
    return Decimal.from_float(time.time()) + Decimal(0)


print("[+] Check formats")

timestamp = getIntTimestamp()
token = base64.b32encode(str(timestamp).encode()).decode()
check(timestamp, token, description="Int timestamp with base32")

timestamp = getIntTimestamp()
token = base64.b64encode(str(timestamp).encode()).decode()
check(timestamp, token, description="Int timestamp with base64")

timestamp = getIntTimestamp()
token = urlencode(str(timestamp))
check(timestamp, token, description="Int timestamp with urlencode")

timestamp = getIntTimestamp()
token = hex(timestamp)[2:]
check(timestamp, token, description="Int timestamp with hexint")

timestamp = getIntTimestamp()
token = hexlify(str(timestamp).encode()).decode()
check(timestamp, token, description="Int timestamp with hexstr")

timestamp = getIntTimestamp()
token = str(uniqid(Decimal(timestamp)))
check(
    timestamp,
    token,
    description="Int timestamp with uniqid",
)

timestamp = getIntTimestamp()
mongoOID_example = "65b23087d5888f1392d74c95"
u = MongoDBObjectID(mongoOID_example)
u.set_timestamp(timestamp)
token = str(u)
check(timestamp, token, description="Int timestamp with mongodb_objectid")

timestamp = getIntTimestamp()
u = uuid.uuid1()
token = str(uuid1(u.node, u.clock_seq, Decimal(timestamp)))
check(timestamp, token, description="Int timestamp with uuidv1")

timestamp = getIntTimestamp()
token = base64.b64encode(hex(timestamp)[2:].encode()).decode()
check(timestamp, token, description="Int timestamp with hexint and base64")

timestamp = getIntTimestamp()
token = base64.b64encode(hexlify(str(timestamp).encode())).decode()
check(timestamp, token, description="Int timestamp with hexstr and base64")

timestamp = getFloatTimestamp()
token = base64.b64encode(uniqid(timestamp).encode()).decode()
check(timestamp, token, description="Float timestamp with uniqid and base64")

timestamp = getFloatTimestamp()
token = hexlify(str(timestamp).encode()).decode()
check(timestamp, token, description="Float timestamp with hexstr")

timestamp = getFloatTimestamp()
token = base64.b64encode(hexlify(str(timestamp).encode())).decode()
check(timestamp, token, description="Float timestamp with hexstr and base64")

timestamp = getFloatTimestamp()
token = uniqid(timestamp)
check(timestamp, token, description="Float timestamp with uniqid")


print("[+] Inmport/Export checks")

timestamp = getIntTimestamp()
u = uuid.uuid1()
token = str(uuid1(u.node, u.clock_seq, Decimal(timestamp)))
token2 = str(uuid1(u.node, u.clock_seq + 1, Decimal(timestamp)))
tolkien = ResetTolkien(token=token, formats=["uuidv1"], alternative_tokens=[token2])
tokens = list(
    tolkien.generate_possible_token(timestamp, range_limit=1, formats=tolkien.formats)
)
success = token2 in token2
print(f"Alternative tokens with uuidv1 : {(OK if success else NOK)}")

timestamp = getIntTimestamp()
token = base64.b64encode(
    hashlib.sha1(str(timestamp).encode()).hexdigest().encode()
).decode()

tolkien = ResetTolkien(token=token, formats=["base64", "sha1"])
success = token == tolkien.encode(str(timestamp))
print(f"Format importation : {(OK if success else NOK)}")

tokens = list(
    tolkien.generate_possible_token(timestamp, range_limit=4, formats=tolkien.formats)
)
success = len(tokens) == 4 and len(set(tokens)) == len(tokens) and tokens[0][0] == token
print(f"Possible token exportation : {(OK if success else NOK)}")

print("[+] Check prefix/suffix")

timestamp = getIntTimestamp()
timestamp_input = timestamp - TIMEDELTA_WITH_INT_VALUE
value = str(timestamp)
token = hashlib.md5(value.encode()).hexdigest()
check(
    timestamp,
    token,
    description="Check prefix/suffix from native value with md5",
    timestamp_input=timestamp_input,
    prefixes=["hello", "you"],
    suffixes=["1", "2"],
)

timestamp = getIntTimestamp()
timestamp_input = timestamp - TIMEDELTA_WITH_INT_VALUE
value = "%s%s" % ("you", str(timestamp))
token = hashlib.md5(value.encode()).hexdigest()
check(
    timestamp,
    token,
    description="Check prefix with md5",
    timestamp_input=timestamp_input,
    prefixes=["hello", "you"],
    suffixes=["1", "2"],
)

timestamp = getIntTimestamp()
timestamp_input = timestamp - TIMEDELTA_WITH_INT_VALUE
value = "%s%s" % (str(timestamp), "2")
token = hashlib.md5(value.encode()).hexdigest()
check(
    timestamp,
    token,
    description="Check suffix with md5",
    timestamp_input=timestamp_input,
    prefixes=["hello", "you"],
    suffixes=["1", "2"],
)

print("[+] Check datetime")

date = datetime.datetime.now(tz=datetime.timezone(datetime.timedelta(hours=-7)))
timestamp = Decimal.from_float(date.timestamp())
timestamp_input = timestamp - TIMEDELTA_WITH_FLOAT_VALUE
token = date.strftime("%a, %d %b %Y %H:%M:%S %Z")
check(
    timestamp,
    token,
    description="Datetime RFC2822 with timezone",
    timestamp_input=timestamp_input,
    date_format_of_token="%a, %d %b %Y %H:%M:%S %Z",
    timezone=-7,
)

date = datetime.datetime.now(tz=datetime.timezone(datetime.timedelta(hours=-7)))
timestamp = Decimal.from_float(date.timestamp())
timestamp_input = timestamp - TIMEDELTA_WITH_FLOAT_VALUE
d = date.strftime("%a, %d %b %Y %H:%M:%S -0700")
token = hashlib.sha1(str(d).encode()).hexdigest()
check(
    timestamp,
    token,
    description="Datetime RFC2822 with timezone and sha1",
    timestamp_input=timestamp_input,
    timezone=-7,
)

date = datetime.datetime.now(tz=datetime.timezone.utc)
timestamp = Decimal.from_float(date.timestamp())
timestamp_input = timestamp - TIMEDELTA_WITH_FLOAT_VALUE
token = base64.b64encode(date.isoformat().encode()).decode()
check(
    timestamp,
    token,
    description="Datetime with base64",
    timestamp_input=timestamp_input,
    date_format_of_token="%Y-%m-%dT%H:%M:%S.%f+00:00",
)

print("[+] Check hashes")

timestamp = getIntTimestamp()
timestamp_input = timestamp + TIMEDELTA_WITH_INT_VALUE
token = hashlib.md5(str(timestamp).encode()).hexdigest()
check(
    timestamp,
    token,
    description="Int timestamp with md5 for token after the timestamp",
    timestamp_input=timestamp_input,
)

timestamp = getIntTimestamp()
timestamp_input = timestamp - TIMEDELTA_WITH_INT_VALUE
token = base64.b64encode(
    hashlib.md5(str(timestamp).encode()).hexdigest().encode()
).decode()
check(
    timestamp,
    token,
    description="Int timestamp with md5 and base64",
    timestamp_input=timestamp_input,
)

timestamp = getIntTimestamp()
timestamp_input = timestamp - TIMEDELTA_WITH_INT_VALUE
token = hexlify(hashlib.md5(str(timestamp).encode()).hexdigest().encode()).decode()
check(
    timestamp,
    token,
    description="Int timestamp with md5 and hexstr",
    timestamp_input=timestamp_input,
)

timestamp = getIntTimestamp()
timestamp_input = timestamp - TIMEDELTA_WITH_INT_VALUE
token = hashlib.md5(str(timestamp).encode()).hexdigest()
check(
    timestamp,
    token,
    description="Int timestamp with md5",
    timestamp_input=timestamp_input,
)

timestamp = getIntTimestamp()
timestamp_input = timestamp - TIMEDELTA_WITH_INT_VALUE
token = hashlib.sha1(str(timestamp).encode()).hexdigest()
check(
    timestamp,
    token,
    description="Int timestamp with sha1",
    timestamp_input=timestamp_input,
)

timestamp = getIntTimestamp()
timestamp_input = timestamp - TIMEDELTA_WITH_INT_VALUE
token = hashlib.sha224(str(timestamp).encode()).hexdigest()
check(
    timestamp,
    token,
    description="Int timestamp with sha224",
    timestamp_input=timestamp_input,
)

timestamp = getIntTimestamp()
timestamp_input = timestamp - TIMEDELTA_WITH_INT_VALUE
token = hashlib.sha256(str(timestamp).encode()).hexdigest()
check(
    timestamp,
    token,
    description="Int timestamp with sha256",
    timestamp_input=timestamp_input,
)

timestamp = getIntTimestamp()
timestamp_input = timestamp - TIMEDELTA_WITH_INT_VALUE
token = hashlib.sha384(str(timestamp).encode()).hexdigest()
check(
    timestamp,
    token,
    description="Int timestamp with sha384",
    timestamp_input=timestamp_input,
)

timestamp = getIntTimestamp()
timestamp_input = timestamp - TIMEDELTA_WITH_INT_VALUE
token = hashlib.sha512(str(timestamp).encode()).hexdigest()
check(
    timestamp,
    token,
    description="Int timestamp with sha512",
    timestamp_input=timestamp_input,
)

timestamp = getIntTimestamp()
timestamp_input = timestamp - TIMEDELTA_WITH_INT_VALUE
token = hashlib.sha3_224(str(timestamp).encode()).hexdigest()
check(
    timestamp,
    token,
    description="Int timestamp with sha3_224",
    timestamp_input=timestamp_input,
)

timestamp = getIntTimestamp()
timestamp_input = timestamp - TIMEDELTA_WITH_INT_VALUE
token = hashlib.sha3_256(str(timestamp).encode()).hexdigest()
check(
    timestamp,
    token,
    description="Int timestamp with sha3_256",
    timestamp_input=timestamp_input,
)

timestamp = getIntTimestamp()
timestamp_input = timestamp - TIMEDELTA_WITH_INT_VALUE
token = hashlib.sha3_384(str(timestamp).encode()).hexdigest()
check(
    timestamp,
    token,
    description="Int timestamp with sha3_384",
    timestamp_input=timestamp_input,
)

timestamp = getIntTimestamp()
timestamp_input = timestamp - TIMEDELTA_WITH_INT_VALUE
token = hashlib.sha3_512(str(timestamp).encode()).hexdigest()
check(
    timestamp,
    token,
    description="Int timestamp with sha3_512",
    timestamp_input=timestamp_input,
)

timestamp = getIntTimestamp()
timestamp_input = timestamp - TIMEDELTA_WITH_INT_VALUE
token = hashlib.blake2s(str(timestamp).encode()).hexdigest()
check(
    timestamp,
    token,
    description="Int timestamp with blake_256",
    timestamp_input=timestamp_input,
)

timestamp = getIntTimestamp()
timestamp_input = timestamp - TIMEDELTA_WITH_INT_VALUE
token = hashlib.blake2b(str(timestamp).encode()).hexdigest()
check(
    timestamp,
    token,
    description="Int timestamp with blake_512",
    timestamp_input=timestamp_input,
)

timestamp = getFloatTimestamp()
timestamp_input = timestamp + TIMEDELTA_WITH_FLOAT_VALUE
u = uuid.uuid1()
token = str(uuid1(u.node, u.clock_seq, timestamp))
check(
    timestamp,
    token,
    description="Float timestamp with uuidv1",
    timestamp_input=timestamp_input,
)

timestamp = getFloatTimestamp()
timestamp_input = timestamp + TIMEDELTA_WITH_FLOAT_VALUE
token = hashlib.blake2b(str(timestamp).encode()).hexdigest()
check(
    timestamp,
    token,
    description="Float timestamp with blake_512",
    timestamp_input=timestamp_input,
)

timestamp = getFloatTimestamp()
timestamp_input = timestamp + TIMEDELTA_WITH_FLOAT_VALUE
token = hashlib.sha1(uniqid(timestamp).encode()).hexdigest()
check(
    timestamp,
    token,
    description="Uniqid timestamp with sha1",
    timestamp_input=timestamp_input,
)

timestamp = getFloatTimestamp()
timestamp_input = timestamp - TIMEDELTA_WITH_FLOAT_VALUE
value = "%s%s%s" % ("you", str(timestamp), "2")
token = hashlib.md5(value.encode()).hexdigest()
check(
    timestamp,
    token,
    description="Check prefix/suffix from float value with md5",
    timestamp_input=timestamp_input,
    prefixes=["hello", "you"],
    suffixes=["1", "2"],
)

date = datetime.datetime.now(tz=datetime.timezone.utc)
timestamp = Decimal.from_float(date.timestamp())
timestamp_input = timestamp - TIMEDELTA_WITH_FLOAT_VALUE
token = hashlib.md5(date.isoformat().encode()).hexdigest()
check(
    timestamp,
    token,
    description="Datetime with md5",
    timestamp_input=timestamp_input,
    date_format_of_token="%Y-%m-%dT%H:%M:%S.%f+00:00",
)

print("[+] Check not working")

timestamp = getFloatTimestamp()
timestamp_input = timestamp + TIMEDELTA_WITH_FLOAT_VALUE
token = hashlib.sha3_256("fusbevuisbevuiesbvuiesbvsie".encode()).hexdigest()
check(
    timestamp,
    token,
    description="Unknown token with sha3_256",
    timestamp_input=timestamp_input,
)



# -*- coding: utf-8 -*-
# Author: Aethlios <tom.chambaretaud@protonmail.com>

import hashlib


class Hashes:
    """Utility class for hashing all available formats"""

    @staticmethod
    def md5(value: str) -> str:
        return hashlib.md5(value.encode()).hexdigest()

    @staticmethod
    def sha1(value: str) -> str:
        return hashlib.sha1(value.encode()).hexdigest()

    @staticmethod
    def sha224(value: str) -> str:
        return hashlib.sha224(value.encode()).hexdigest()

    @staticmethod
    def sha256(value: str) -> str:
        return hashlib.sha256(value.encode()).hexdigest()

    @staticmethod
    def sha384(value: str) -> str:
        return hashlib.sha384(value.encode()).hexdigest()

    @staticmethod
    def sha512(value: str) -> str:
        return hashlib.sha512(value.encode()).hexdigest()

    @staticmethod
    def sha3_224(value: str) -> str:
        return hashlib.sha3_224(value.encode()).hexdigest()

    @staticmethod
    def sha3_256(value: str) -> str:
        return hashlib.sha3_256(value.encode()).hexdigest()

    @staticmethod
    def sha3_384(value: str) -> str:
        return hashlib.sha3_384(value.encode()).hexdigest()

    @staticmethod
    def sha3_512(value: str) -> str:
        return hashlib.sha3_512(value.encode()).hexdigest()

    @staticmethod
    def blake_256(value: str) -> str:
        return hashlib.blake2s(value.encode()).hexdigest()

    @staticmethod
    def blake_512(value: str) -> str:
        return hashlib.blake2b(value.encode()).hexdigest()
